needs "metis_tactic.ml";;

(* Tests for the Metis tactic. *)

g `(!x:nat. P x) ==> P y`;;
e (Metis_tactic.FOL_PREPARE_TAC []);;
e (Metis_tactic.PURE_METIS_TAC (Timeout 5.0));;

g `a \/ b ==> ~a \/ b ==> b`;;
e (REPEAT DISCH_TAC);;
e (REFUTE_THEN ASSUME_TAC);;
e (Metis_tactic.PURE_METIS_TAC (Timeout 5.0));;

(* not provable by MESON (in reasonable time), but by Metis *)
g `(!x y. m(x,y) = m(y,x)) ==>
   (!x y z. m(x, m(y,z)) = m(m(x,y), z)) ==>
   m(a, m(b, m(c, m(d, m(e, f))))) = m(f, m(e, m(d, m(c, m(b, a)))))`;;
e (Metis_tactic.METIS_TAC []);;

g `(!x y. m(x,y) = m(y,x)) ==>
   (!x y z. m(x, m(y,z)) = m(m(x,y), z)) ==>
   m(a, m(b, c)) = m(c, m(b, a))`;;
e (Metis_tactic.METIS_TAC []);;

g `(!x y. m x y = m y x) ==>
   (!x y z. m x (m y z) = m (m x y) z) ==>
   m a (m b c) = m c (m b a)`;;
e (Metis_tactic.METIS_TAC []);;


(* unprovable by MESON, but also unprovable by Metis so far
   (satisfiable assignment) *)
let dep = `!t. (\x. t x) = (t:'a->'b)`;;
let goal = `!f g. (!x. f x = g x) ==> (f:'a->'b) = g`;;
g (mk_imp (dep, goal));;


(* provable by both MESON and Metis *)
let goal = `!P x. (!y. P y <=> y = x) ==> (@) (P:'a->bool) = x`;;
let dep1 = `(?) = (\P. !q. (!x. (P:'a->bool) x ==> q) ==> q)`;;
let dep2 = `(?) = (\P. (P:'a->bool) ((@) P))`;;
g (mk_imp (dep1, mk_imp (dep2, goal)));;
e (Metis_tactic.METIS_TAC []);;


(* provable by both MESON and Metis *)
g `(!x y. (f:nat->nat) x = (g:nat->nat) y) ==> (!x y. g y = f x)`;;



(* get the current ths of a goalstate *)
let goalstate_ths gs =
  let goals = (fun (x, l, f) -> l) (hd gs) in
  let goal = hd goals in
  map snd (fst goal)
;;

