{
  open Tptp_parse;;
}

let white = [' ' '\t']
let newline = ['\r' '\n']
let letter = ['/' 'a'-'z' 'A'-'Z' '0'-'9' '_' '-']
let letterdot = ['/' 'a'-'z' 'A'-'Z' '0'-'9' '_' '-' '.']
let any = [^ '\r' '\n']

rule lex_find_begin = parse
| _+ "SZS output start" any+ {()}

and lex_tptp = parse
| white+             {lex_tptp lexbuf}
| '%' any*           {lex_tptp lexbuf}
| '#' any*           {lex_tptp lexbuf}
| newline            {let pos = lexbuf.Lexing.lex_curr_p in
                     lexbuf.Lexing.lex_curr_p <- { pos with
                       Lexing.pos_lnum = pos.Lexing.pos_lnum + 1;
                       Lexing.pos_bol = pos.Lexing.pos_cnum;
                     }; lex_tptp lexbuf}
| eof                {Eof}
| "SZS output end"   {Eof}
| "# SZS output end" {Eof}
| "% SZS output end" {Eof}
| '('                {(*Printf.printf "(%!"; *)Openp}
| ')'                {(*Printf.printf ")%!"; *)Closep}
| '['                {Opens}
| ']'                {Closes}
| '.'                {Dot}
| ','                {Comma}
| '!' white* '['     {(*Printf.printf "!%!"; *)All (lex_ids lexbuf)}
| '?' white* '['     {Ex (lex_ids lexbuf)}
| '='                {Eq}
| "!="               {Neq}
| '~'                {Tilde}
| "<=>"              {Eqvt}
| "<~>"              {Neqvt}
| "=>"               {(*Printf.printf ">%!"; *)Impl}
| "<="               {(*Printf.printf ">%!"; *)Revimpl}
| '&'                {(*Printf.printf "&%!"; *)And}
| '|'                {Or}
| '$' letter+        {Word (Lexing.lexeme lexbuf)}
| letter letterdot*  {let s = Lexing.lexeme lexbuf in Word s }
| '\''               {Word ("'" ^ lex_squot lexbuf ^ "'")}
| '"'                {Word ("\"" ^ lex_dquot lexbuf ^ "\"")}

and lex_squot = parse
| '\''               {""}
| '\\' '\\'          {"\\" ^ lex_squot lexbuf}
| '\\' '\''          {"'" ^ lex_squot lexbuf}
| [^ '\'' '\\']+     {let s = Lexing.lexeme lexbuf in s ^ lex_squot lexbuf}
| _                  {failwith "lex_squot: invalid escaped sequence"}

and lex_dquot = parse
| '"'                {""}
| '\\' '\\'          {"\\" ^ lex_dquot lexbuf}
| '\\' '"'           {"\"" ^ lex_dquot lexbuf}
| [^ '"' '\\']+      {let s = Lexing.lexeme lexbuf in s ^ lex_dquot lexbuf}
| _                  {failwith "lex_dquot: invalid escaped sequence"}

and lex_ids = parse
| white+             {lex_ids lexbuf}
| letter+            {let i = Lexing.lexeme lexbuf in
                      i :: (lex_ids_more lexbuf)}

and lex_ids_more = parse
| white+             {lex_ids_more lexbuf}
| ','                {lex_ids lexbuf}
| ']' white* ':'     {[]}

{
}
