%{
  open Tptp;;
  let is_uppercase s = s <> "" && let c = Char.code s.[0] in (c >= 65 && c <= 90);;
%}

%token <string> Word
%token <string list> All
%token <string list> Ex
%token Cnf Fof Tff Thf Tt
%token Eof Openp Closep Dot Comma Eq Neq Tilde Eqvt Neqvt Impl Revimpl And Or Opens Closes
%right Impl
%nonassoc Eqvt
%nonassoc Eq Neq
%right Or
%left And
%nonassoc Tilde
%nonassoc All Ex

%start parse_tstp parse_tptp
%type <(string * string * string * Tptp.fof_form * (string * string list))> parse_tstp
%type <(string * string * string * Tptp.fof_form)> parse_tptp

%%

parse_tstp :
  Word Openp Word Comma Word Comma formula Comma just Closep Dot { ($1, $3, $5, $7, $9) }
| Word Openp Word Comma Word Comma formula Comma just Comma Opens Word Closes Closep Dot { ($1, $3, $5, $7, $9) }
| Eof { raise End_of_file };

parse_tptp :
  Word Openp Word Comma Word Comma formula Closep Dot { ($1, $3, $5, $7) }
| Eof { raise End_of_file };

formula :
| qformula     { $1 }
| formula Impl formula { Disj (Neg $1, $3) }
| formula Revimpl formula { Disj (Neg $3, $1) }
| formula Eqvt formula  { Eqiv ($1, $3) }
| formula And formula { Conj ($1, $3) }
| formula Or formula { Disj ($1, $3) }
| formula Neqvt formula { Neg (Eqiv ($1, $3)) }

qformula :
  Word { save_arity ($1,[]); Atom($1, []) }
| Word Openp ts { save_arity ($1, $3); Atom($1, $3) }
| fterm Eq fterm { Atom ("=", [$1; $3]) }
| fterm Neq fterm { Neg (Atom ("=", [$1; $3])) }
| Tilde qformula { Neg $2 }
| All qformula { fof_list_forall (List.map var $1) $2 }
| Ex qformula { fof_list_exists (List.map var $1) $2 }
| Openp formula Closep { $2 }

fterm :
  Word { if is_uppercase $1 then V (var $1) else (save_arity ($1, []); A($1, [])) }
| Word Openp ts { save_arity ($1, $3); A($1, $3) }

ts :
  fterm Comma ts { $1 :: $3 }
| fterm Closep   { [$1] };

flist :
  formula Comma flist  { $1 :: $3 }
| formula Closep { [$1] }
| Closep { [] };

just :
  Word   { ("", [$1]) }
| Word Openp Word Comma Word Closep { if $1 = "file" then ($3, [$5]) else failwith "j1" }
| Word Openp Word Comma Opens Word Openp Word Closep Closes Comma Opens parents Closep
         { if $1 = "inference" then ($3, $13) else failwith "j2" };

parents :
  parent Comma parents { $1 @ $3 }
| parent Closes   { $1 }
| Closes        { [] }

parent:
  Word     { [$1] }
| Word Openp Word Closep { if $1 = "theory" then [] else failwith "j3" }
| Word Openp Word Comma Opens Word Closes Closep { if $1 = "theory" then [] else failwith "j4" }
| Word Openp Word Comma Opens Word Closes Closep { if $1 = "theory" then [] else failwith "j5" }
| Word Openp Word Comma Opens Word Openp Word Closep Closes Comma Opens parents Closep
   { if $1 = "inference" then $13 else failwith "j6" }

